unit TestADOEventsUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ADODB_TLB, ADOEvents, OleCtrls;

type
  TTestADOEventsForm = class(TForm)
    ConnectionTestButton: TButton;
    ListBox1: TListBox;
    RecordsetTestButton: TButton;
    ADOConnectionEvents1: TADOConnectionEvents;
    ADORecordsetEvents1: TADORecordsetEvents;

    // Connection event handlers
    procedure ConnectionTestButtonClick(Sender: TObject);
    procedure RecordsetTestButtonClick(Sender: TObject);
    procedure ADOConnectionEvents1BeginTransComplete(Sender: TObject;
      TransactionLevel: Integer; const pError: Error;
      var adStatus: TOleEnum; const pConnection: _Connection);
    procedure ADOConnectionEvents1CommitTransComplete(Sender: TObject;
      const pError: Error; var adStatus: TOleEnum;
      const pConnection: _Connection);
    procedure ADOConnectionEvents1ConnectComplete(Sender: TObject;
      const pError: Error; var adStatus: TOleEnum;
      const pConnection: _Connection);
    procedure ADOConnectionEvents1Disconnect(Sender: TObject;
      var adStatus: TOleEnum; const pConnection: _Connection);
    procedure ADOConnectionEvents1ExecuteComplete(Sender: TObject;
      RecordsAffected: Integer; const pError: Error;
      var adStatus: TOleEnum; const pCommand: _Command;
      const pRecordset: _Recordset; const pConnection: _Connection);
    procedure ADOConnectionEvents1InfoMessage(Sender: TObject;
      const pError: Error; var adStatus: TOleEnum;
      const pConnection: _Connection);
    procedure ADOConnectionEvents1RollbackTransComplete(Sender: TObject;
      const pError: Error; var adStatus: TOleEnum;
      const pConnection: _Connection);
    procedure ADOConnectionEvents1WillConnect(Sender: TObject;
      var ConnectionString, UserID, Password: WideString;
      var Options: Integer; var adStatus: TOleEnum;
      const pConnection: _Connection);
    procedure ADOConnectionEvents1WillExecute(Sender: TObject;
      var Source: WideString; var CursorType, LockType: TOleEnum;
      var Options: Integer; var adStatus: TOleEnum;
      const pCommand: _Command; const pRecordset: _Recordset;
      const pConnection: _Connection);

    // Recordset event handlers
    procedure ADORecordsetEvents1EndOfRecordset(Sender: TObject;
      var fMoreData: WordBool; var adStatus: TOleEnum;
      const pRecordset: _Recordset);
    procedure ADORecordsetEvents1FetchComplete(Sender: TObject;
      const pError: Error; var adStatus: TOleEnum;
      const pRecordset: _Recordset);
    procedure ADORecordsetEvents1FetchProgress(Sender: TObject; Progress,
      MaxProgress: Integer; var adStatus: TOleEnum;
      const pRecordset: _Recordset);
    procedure ADORecordsetEvents1FieldChangeComplete(Sender: TObject;
      cFields: Integer; Fields: OleVariant; const pError: Error;
      var adStatus: TOleEnum; const pRecordset: _Recordset);
    procedure ADORecordsetEvents1MoveComplete(Sender: TObject;
      adReason: TOleEnum; const pError: Error; var adStatus: TOleEnum;
      const pRecordset: _Recordset);
    procedure ADORecordsetEvents1RecordChangeComplete(Sender: TObject;
      adReason: TOleEnum; cRecords: Integer; const pError: Error;
      var adStatus: TOleEnum; const pRecordset: _Recordset);
    procedure ADORecordsetEvents1RecordsetChangeComplete(Sender: TObject;
      adReason: TOleEnum; const pError: Error; var adStatus: TOleEnum;
      const pRecordset: _Recordset);
    procedure ADORecordsetEvents1WillChangeField(Sender: TObject;
      cFields: Integer; Fields: OleVariant; var adStatus: TOleEnum;
      const pRecordset: _Recordset);
    procedure ADORecordsetEvents1WillChangeRecord(Sender: TObject;
      adReason: TOleEnum; cRecords: Integer; var adStatus: TOleEnum;
      const pRecordset: _Recordset);
    procedure ADORecordsetEvents1WillChangeRecordset(Sender: TObject;
      adReason: TOleEnum; var adStatus: TOleEnum;
      const pRecordset: _Recordset);
    procedure ADORecordsetEvents1WillMove(Sender: TObject;
      adReason: TOleEnum; var adStatus: TOleEnum;
      const pRecordset: _Recordset);
    procedure FormDestroy(Sender: TObject);
  private
    FADOConnection : _Connection;
    FADORecordset  : _Recordset;
  end;

var
  TestADOEventsForm: TTestADOEventsForm;

implementation

{$R *.DFM}

procedure TTestADOEventsForm.ConnectionTestButtonClick(Sender: TObject);
var
  RowsAffected : OleVariant;
begin
  Listbox1.Items.Clear;

  if not Assigned(FADOConnection) then
  begin
    FADOConnection := CoConnection.Create;
    ADOConnectionEvents1.ADOConnection := FADOConnection;
  end;

  FADOConnection.Open('Driver=Microsoft dBase Driver (*.dbf);' +
                      'DefaultDir=F:\Program Files\Common Files\Borland Shared\Data',
                      '', '', -1);
  try
    // if FADOConnection.BeginTrans <> adErrInTransaction then
    try
      FADORecordSet := FADOConnection.Execute('SELECT * FROM Animals', RowsAffected, -1);
      // FADOConnection.CommitTrans;
    except
    // FADOConnection.RollbackTrans;
    end;
  finally
    FADOConnection.Close;
  end;

  if FADORecordset.State <> adStateClosed then
  begin
    FADORecordset.Close;
  end;
end;

procedure TTestADOEventsForm.RecordsetTestButtonClick(Sender: TObject);
begin
  Listbox1.Items.Clear;

  FADORecordSet := CoRecordset.Create;
  ADORecordsetEvents1.ADORecordset := FADORecordset;

  FADORecordset.Open('SELECT * FROM Animals',
                  'Driver=Microsoft dBase Driver (*.dbf);' +
                  'DefaultDir=F:\Program Files\Common Files\Borland Shared\Data',
                  adOpenDynamic, adLockPessimistic, adCmdText);
  try
    if FADORecordset.State = adStateOpen then
    begin
      // Edit test
      FADORecordSet.Fields.Item['Name'].Value := 'Todd!';
      if FADORecordSet.EditMode = adEditInProgress then
      begin
         Listbox1.Items.Add('Comment: Edit In Progress');
      end;

      while not FADORecordset.EOF do
      begin
        FADORecordset.MoveNext;
      end;
    end;
  finally
    if FADORecordset.State <> adStateClosed then
    begin
      FADORecordset.Close;
    end;
  end;
end;

//----------------------------------------------------------------
//  Event handlers for an ADO Connection
//----------------------------------------------------------------

procedure TTestADOEventsForm.ADOConnectionEvents1BeginTransComplete(
  Sender: TObject; TransactionLevel: Integer; const pError: Error;
  var adStatus: TOleEnum; const pConnection: _Connection);
const
  EVEMT_TEST_OUTPUT_STR1 = 'BeginTransComplete: Error MSg(%s) status(%d) ADO version(%s)';
  EVEMT_TEST_OUTPUT_STR2 = 'BeginTransComplete: status(%d) ADO version(%s) TransactionLevel: (%d)';
begin
  if Assigned(pError) then
  begin
    Listbox1.Items.Add(Format(EVEMT_TEST_OUTPUT_STR1, [pError.Description, adStatus, pConnection.Version]));
  end else
  begin
    Listbox1.Items.Add(Format(EVEMT_TEST_OUTPUT_STR2, [adStatus, pConnection.Version, TransactionLevel]));
  end;
end;

procedure TTestADOEventsForm.ADOConnectionEvents1CommitTransComplete(
  Sender: TObject; const pError: Error; var adStatus: TOleEnum;
  const pConnection: _Connection);
const
  EVEMT_TEST_OUTPUT_STR1 = 'CommitTransComplete: Error MSg(%s) status: (%d) ADO version(%s)';
  EVEMT_TEST_OUTPUT_STR2 = 'CommitTransComplete: status(%d) ADO version(%s)';
begin
  if Assigned(pError) then
  begin
    Listbox1.Items.Add(Format(EVEMT_TEST_OUTPUT_STR1, [pError.Description, adStatus, pConnection.Version]));
  end else
  begin
    Listbox1.Items.Add(Format(EVEMT_TEST_OUTPUT_STR2, [adStatus, pConnection.Version]));
  end;
end;

procedure TTestADOEventsForm.ADOConnectionEvents1ConnectComplete(
  Sender: TObject; const pError: Error; var adStatus: TOleEnum;
  const pConnection: _Connection);
const
  EVEMT_TEST_OUTPUT_STR1 = 'ConnectComplete: Error MSg(%s) status(%d) ADO version(%s)';
  EVEMT_TEST_OUTPUT_STR2 = 'ConnectComplete: status(%d) ADO version(%s)';
begin
  if Assigned(pError) then
  begin
    Listbox1.Items.Add(Format(EVEMT_TEST_OUTPUT_STR1, [pError.Description, adStatus, pConnection.Version]));
  end else
  begin
    Listbox1.Items.Add(Format(EVEMT_TEST_OUTPUT_STR2, [adStatus, pConnection.Version]));
  end;
end;

procedure TTestADOEventsForm.ADOConnectionEvents1Disconnect(
  Sender: TObject; var adStatus: TOleEnum; const pConnection: _Connection);
const
  EVEMT_TEST_OUTPUT_STR = 'Disconnect: Status(%d) ADO version(%s)';
begin
  Listbox1.Items.Add(Format(EVEMT_TEST_OUTPUT_STR, [adStatus, pConnection.Version]));
end;

procedure TTestADOEventsForm.ADOConnectionEvents1ExecuteComplete(
  Sender: TObject; RecordsAffected: Integer; const pError: Error;
  var adStatus: TOleEnum; const pCommand: _Command;
  const pRecordset: _Recordset; const pConnection: _Connection);
const
  EVEMT_TEST_OUTPUT_STR1 = 'ExecuteComplete: Error MSg(%s) status: (%d) ADO version(%s)';
  EVEMT_TEST_OUTPUT_STR2 = 'ExecuteComplete: status(%d) ADO version(%s) Command Sring(%s) Records affected(%d)';
begin
  if Assigned(pError) then
  begin
    Listbox1.Items.Add(Format(EVEMT_TEST_OUTPUT_STR1, [pError.Description, adStatus, pConnection.Version]));
  end else
  begin
    Listbox1.Items.Add(Format(EVEMT_TEST_OUTPUT_STR2, [adStatus, pConnection.Version, pCommand.CommandText, RecordsAffected]));
  end;
end;

procedure TTestADOEventsForm.ADOConnectionEvents1InfoMessage(
  Sender: TObject; const pError: Error; var adStatus: TOleEnum;
  const pConnection: _Connection);
const
  EVEMT_TEST_OUTPUT_STR1 = 'InfoMessage: Error MSg(%s) status(%d) ADO version(%s)';
  EVEMT_TEST_OUTPUT_STR2 = 'InfoMessage: status(%d) ADO version(%s)';
begin
  if Assigned(pError) then
  begin
    Listbox1.Items.Add(Format(EVEMT_TEST_OUTPUT_STR1, [pError.Description, adStatus, pConnection.Version]));
  end else
  begin
    Listbox1.Items.Add(Format(EVEMT_TEST_OUTPUT_STR2, [adStatus, pConnection.Version]));
  end;
end;

procedure TTestADOEventsForm.ADOConnectionEvents1RollbackTransComplete(
  Sender: TObject; const pError: Error; var adStatus: TOleEnum;
  const pConnection: _Connection);
const
  EVEMT_TEST_OUTPUT_STR1 = 'RollbackTransComplete: Error MSg(%s) status: (%d)d ADO version(%s)';
  EVEMT_TEST_OUTPUT_STR2 = 'RollbackTransComplete: status: (%d) ADO version(%s)';
begin
  if Assigned(pError) then
  begin
    Listbox1.Items.Add(Format(EVEMT_TEST_OUTPUT_STR1, [pError.Description, adStatus, pConnection.Version]));
  end else
  begin
    Listbox1.Items.Add(Format(EVEMT_TEST_OUTPUT_STR2, [adStatus, pConnection.Version]));
  end;
end;

procedure TTestADOEventsForm.ADOConnectionEvents1WillConnect(
  Sender: TObject; var ConnectionString, UserID, Password: WideString;
  var Options: Integer; var adStatus: TOleEnum;
  const pConnection: _Connection);
const
  EVEMT_TEST_OUTPUT_STR = 'WillConnect: USerID(%s) Password(%s) Status(%d) ADO version(%s)';
begin
  Listbox1.Items.Add(Format(EVEMT_TEST_OUTPUT_STR, [UserID, Password, adStatus, pConnection.Version]));
//  adStatus := adStatusCancel;
end;

procedure TTestADOEventsForm.ADOConnectionEvents1WillExecute(
  Sender: TObject; var Source: WideString; var CursorType,
  LockType: TOleEnum; var Options: Integer; var adStatus: TOleEnum;
  const pCommand: _Command; const pRecordset: _Recordset;
  const pConnection: _Connection);
const
  EVEMT_TEST_OUTPUT_STR = 'WillExecute: Status(%d) ADO version(%s) Source(%s) CursorType(%d) LockType(%d) Options(%d)';
begin
//pCommand may be Nil.
  Listbox1.Items.Add(Format(EVEMT_TEST_OUTPUT_STR, [adStatus, pConnection.Version, Source, CursorType, LockType, Options]));
end;

//----------------------------------------------------------------
//  Event handlers for an ADO Recordset
//----------------------------------------------------------------

procedure TTestADOEventsForm.ADORecordsetEvents1EndOfRecordset(
  Sender: TObject; var fMoreData: WordBool; var adStatus: TOleEnum;
  const pRecordset: _Recordset);
const
  EVEMT_TEST_OUTPUT_STR = 'EndOfRecordset: status(%d) More data(%d) Recordset State(%d)';

begin
  Listbox1.Items.Add(Format(EVEMT_TEST_OUTPUT_STR, [adStatus, Word(fMoreData), pRecordset.State]));
end;

procedure TTestADOEventsForm.ADORecordsetEvents1FetchComplete(
  Sender: TObject; const pError: Error; var adStatus: TOleEnum;
  const pRecordset: _Recordset);
const
  EVEMT_TEST_OUTPUT_STR1 = 'FetchComplete: Error(%s) Status(%d) Recordset State(%d)';
  EVEMT_TEST_OUTPUT_STR2 = 'FetchComplete: Status(%d) Recordset State(%d)';
begin
  if Assigned(pError) then
  begin
    Listbox1.Items.Add(Format(EVEMT_TEST_OUTPUT_STR1, [pError.Description, adStatus, pRecordset.State]));
  end else
  begin
    Listbox1.Items.Add(Format(EVEMT_TEST_OUTPUT_STR2, [adStatus, pRecordset.State]));
  end;
end;

procedure TTestADOEventsForm.ADORecordsetEvents1FetchProgress(
  Sender: TObject; Progress, MaxProgress: Integer; var adStatus: TOleEnum;
  const pRecordset: _Recordset);
const
  EVEMT_TEST_OUTPUT_STR = 'FetchProgress: status(%d) Progress(%d) Max progress(%d) Recordset State(%d)';
begin
  Listbox1.Items.Add(Format(EVEMT_TEST_OUTPUT_STR, [adStatus, Progress, MaxProgress, pRecordset.State]));
end;

procedure TTestADOEventsForm.ADORecordsetEvents1FieldChangeComplete(
  Sender: TObject; cFields: Integer; Fields: OleVariant;
  const pError: Error; var adStatus: TOleEnum;
  const pRecordset: _Recordset);
const
  EVEMT_TEST_OUTPUT_STR1 = 'FieldChangeComplete: Error(%s) status(%d) Field count(%d) Recordset State(%d)';
  EVEMT_TEST_OUTPUT_STR2 = 'FieldChangeComplete: status(%d) Field count(%d) Recordset State(%d)';
begin
  if Assigned(pError) then
  begin
    Listbox1.Items.Add(Format(EVEMT_TEST_OUTPUT_STR1, [pError.Description, adStatus, cFields, pRecordset.State]));
  end else
  begin
    Listbox1.Items.Add(Format(EVEMT_TEST_OUTPUT_STR2, [adStatus, cFields, pRecordset.State]));
  end;
end;

procedure TTestADOEventsForm.ADORecordsetEvents1MoveComplete(
  Sender: TObject; adReason: TOleEnum; const pError: Error;
  var adStatus: TOleEnum; const pRecordset: _Recordset);
const
  EVEMT_TEST_OUTPUT_STR1 = 'MoveComplete: Error(%s) Status(%d) Reason(%d) Recordset State(%d)';
  EVEMT_TEST_OUTPUT_STR2 = 'MoveComplete: Status(%d) Reason(%d) Recordset State(%d)';
begin
  if Assigned(pError) then
  begin
    Listbox1.Items.Add(Format(EVEMT_TEST_OUTPUT_STR1, [pError.Description, adStatus, adReason, pRecordset.State]));
  end else
  begin
    Listbox1.Items.Add(Format(EVEMT_TEST_OUTPUT_STR2, [adStatus, adReason, pRecordset.State]));
  end;
end;

procedure TTestADOEventsForm.ADORecordsetEvents1RecordChangeComplete(
  Sender: TObject; adReason: TOleEnum; cRecords: Integer;
  const pError: Error; var adStatus: TOleEnum;
  const pRecordset: _Recordset);
const
  EVEMT_TEST_OUTPUT_STR1 = 'RecordChangeComplete: Error(%s) Status(%d) Reason(%d) Record count(%d) Recordset State(%d)';
  EVEMT_TEST_OUTPUT_STR2 = 'RecordChangeComplete: Status(%d) Reason(%d) Record count(%d) Recordset State(%d)';
begin
  if Assigned(pError) then
  begin
    Listbox1.Items.Add(Format(EVEMT_TEST_OUTPUT_STR1, [pError.Description, adStatus, adReason, cRecords, pRecordset.State]));
  end else
  begin
    Listbox1.Items.Add(Format(EVEMT_TEST_OUTPUT_STR2, [adStatus, adReason, cRecords, pRecordset.State]));
  end;
end;

procedure TTestADOEventsForm.ADORecordsetEvents1RecordsetChangeComplete(
  Sender: TObject; adReason: TOleEnum; const pError: Error;
  var adStatus: TOleEnum; const pRecordset: _Recordset);
const
  EVEMT_TEST_OUTPUT_STR1 = 'RecordsetChangeComplete: Error(%s) Status(%d) Reason(%d) Record count(%d) Recordset State(%d)';
  EVEMT_TEST_OUTPUT_STR2 = 'RecordsetChangeComplete: Status(%d) Reason(%d) Recordset State(%d)';
begin
  if Assigned(pError) then
  begin
    Listbox1.Items.Add(Format(EVEMT_TEST_OUTPUT_STR1, [pError.Description, adStatus, adReason, pRecordset.State]));
  end else
  begin
    Listbox1.Items.Add(Format(EVEMT_TEST_OUTPUT_STR2, [adStatus, adReason, pRecordset.State]));
  end;
end;

procedure TTestADOEventsForm.ADORecordsetEvents1WillChangeField(
  Sender: TObject; cFields: Integer; Fields: OleVariant;
  var adStatus: TOleEnum; const pRecordset: _Recordset);
const
  EVEMT_TEST_OUTPUT_STR = 'WillChangeField: status(%d) Field count(%d) Recordset State(%d)';
begin
  Listbox1.Items.Add(Format(EVEMT_TEST_OUTPUT_STR, [adStatus, cFields, pRecordset.State]));
end;

procedure TTestADOEventsForm.ADORecordsetEvents1WillChangeRecord(
  Sender: TObject; adReason: TOleEnum; cRecords: Integer;
  var adStatus: TOleEnum; const pRecordset: _Recordset);
const
  EVEMT_TEST_OUTPUT_STR = 'WillChangeRecord: status(%d) Reason(%d) Record count(%d) Recordset State(%d)';
begin
  Listbox1.Items.Add(Format(EVEMT_TEST_OUTPUT_STR, [adStatus, adReason, cRecords, pRecordset.State]));
end;

procedure TTestADOEventsForm.ADORecordsetEvents1WillChangeRecordset(
  Sender: TObject; adReason: TOleEnum; var adStatus: TOleEnum;
  const pRecordset: _Recordset);
const
  EVEMT_TEST_OUTPUT_STR = 'WillChangeRecordset: status(%d) Reason(%d) Recordset State(%d)';
begin
  Listbox1.Items.Add(Format(EVEMT_TEST_OUTPUT_STR, [adStatus, adReason, pRecordset.State]));
end;

procedure TTestADOEventsForm.ADORecordsetEvents1WillMove(Sender: TObject;
  adReason: TOleEnum; var adStatus: TOleEnum;
  const pRecordset: _Recordset);
const
  EVEMT_TEST_OUTPUT_STR = 'WillMove: status(%d) Reason(%d) Recordset State(%d)';
begin
  Listbox1.Items.Add(Format(EVEMT_TEST_OUTPUT_STR, [adStatus, adReason, pRecordset.State]));
end;

procedure TTestADOEventsForm.FormDestroy(Sender: TObject);
begin
  ADOConnectionEvents1.ADOConnection := Nil;
  ADORecordsetEvents1.ADORecordset := Nil;
end;

end.
